program BlimDos;

  uses
    GLOBALS, MATHS, UTILS, ACQUIRE, ORGANISE, Dos, Crt, Graph;

  var
    theMenuItem: integer;

begin
  SetTheText;

  Writeln;
  Writeln('     BLIMDOS.PAS');
  Writeln;
  Writeln('     This program  can be used to investigate  band-limited autoregressive');
  Writeln('     processes, of which the  spectral density  functions are supported in');
  Writeln('     the  frequency domain on  an interval that  runs from  zero frequency');
  Writeln('     up to  a limiting value that  is  less  than the Nyquist frequency of');
  Writeln('     pi radians per sampling interval.');
  Writeln;
  Writeln('     The consequences  can be investigated of ingnoring  the limitation in');
  Writeln('     frequency by estimating the parameters of the process via an ordinary');
  Writeln('     autoregressive estimator.');
  Writeln;
  Writeln('     Appropriate  estimates  may be derived by  synthesising  a continuous');
  Writeln('     function  from the Fourier coefficients of the data and by resampling');
  Writeln('     the function at less  than the  original sampling  rate. In this way,');
  Writeln('     the resampled data can be  mapped onto the Nyquist interval, and then');
  Writeln('     an ordinary estimator can be used.');
  Writeln;
  Writeln(' ' : 66, '<RETURN>');
  Readln;

  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;


  allsDone := false;
  noPageParameters := true;
  ARMAModel := false;
  stability := true;
  pseudoData := false;
  empiricData := false;
  ARestimates := false;
  ARMAGen.bandLimits := false;
  ARMAGen.contaminated := false;
  figNumber := 1;
  genSeed := 365;
  seedStore := 377;

  repeat
    ClrScr;
    Writeln;
    Writeln('    YULEWHITE.PAS: Autoregressive Estimation');
    Writeln;
    Writeln('         1. Page Parameters');
    Writeln;
    Writeln('         2. Specify an Autoregressive Model');
    Writeln;
    Writeln('         3. Get the Data and Plot the Data');
    Writeln;
    Writeln('         4. Polynomial Detrending and Lowpass Filtering');
    Writeln;
    Writeln('         5. Plot the Periodogram');
    Writeln;
    Writeln('         6. Autoregressive Estimation');
    Writeln;
    Writeln('         7. Plot the Spectral Density');
    Writeln;
    Writeln('         8. Monte-Carlo Trials: Graphical');
    Writeln;
    Writeln('         9. Monte-Carlo Trials: Numerical');
    Writeln;
    Writeln('         10. SAVE and EXIT');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 10, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);{in UTILS}
      2:
        begin
          RationalAcquisition(ARMAGen, stability);
          ARMAGen.varepsilon := 1.0;
        end;
      3: 
        GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      4: 
        TrendExtract(SCREENBOX, POSTBOX, dataBOX, trendBOX, residueBOX, dataVector, trendVector, residueVec);
      5:
        PlotThePeriodogram(ARMAGen, AREstim, dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      6: 
        AREstimation(ARMAGen, AREstim, dataBOX, dataVector); {in ORGANISE}
      7: 
        PlotTheSpectrum(ARMAGen, AREstim, dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      8: 
        MonteCarloTrials(ARMAGen, AREstim, dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      9: 
        NumericalTrials(ARMAGen, AREstim, dataBOX, dataVector);
      10:
        SaveAndExit(dataBOX, dataVector);
    end;

  until allsDone;
  RevertText;

end. {BLIMDOS Main Unit}
   {________________________________________________}

  {Compile the Units in the Following Order:}

  {1. GLOBALS}
  {2. MATHS}
  {3. UTILS}
  {4. SCREEN}
  {5. POST}
  {6. ROOTS}
  {7. SPECTRAL}
  {8. ACQUIRE}
  {9. AUTOREG}
  {10. DATA}
  {11. ORGANISE}
  {12. MAIN}

