unit MATHS; {Unit 2 of BLIMDOS.PAS}

interface

  uses
    GLOBALS;

  function Even (q: integer): boolean;

  function Max (p, q: integer): integer;

  function Min (p, q: integer): integer;

  function Rmax (x, y: real): real;

  function Rmin (x, y: real): real;

  function Power (x: real;
                  n: integer): real;

  function Tan (theta: real): real;

  function CoTan (theta: real): real;

  function Sign (alpha: real): integer;

  function Arg (psi: complex): real;

  procedure ComplexAdd (a, b: complex;
                  var result: complex);

  procedure ComplexSub (a, b: complex;
                  var result: complex);

  function Cmod (a: complex): real;

  function Cmodsqr (a: complex): real;

  procedure ComplexMult (a, b: complex;
                  var result: complex);

  procedure ComplexDiv (a, b: complex;
                  var result: complex);

  procedure ComplexSqrt (a: complex;
                  var result: complex);

  function TheArgument (thepoint: complex): real;

  function TheModulus (apoint: complex): real;

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);

  procedure QuadraticRoots (var alpha: precisionVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector);

  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x, b: vector);

  procedure Toeplitz (gamma: vector;
                  var y: longVector;
                  n, q: integer);

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

implementation

{_________________________________________________}

  function Even (q: integer): boolean;

  begin
    Even := false;
    if q mod 2 = 0 then
      Even := true;
  end; {Even}

{_________________________________________________}

  function Max (p, q: integer): integer;

  begin
    if p > q then
      Max := p
    else
      Max := q
  end; {Max}

{_________________________________________________}

  function Min (p, q: integer): integer;

  begin
    if p < q then
      Min := p
    else
      Min := q
  end; {Min}

{_________________________________________________}

  function Rmax (x, y: real): real;

  begin
    if x > y then
      Rmax := x
    else
      Rmax := y;
  end; {Rmax}

{_________________________________________________}

  function Rmin (x, y: real): real;

  begin
    if x > y then
      Rmin := y
    else
      Rmin := x;
  end; {Rmin}

{_________________________________________________}

  function Power (x: real;
                  n: integer): real;

    var
      logPower: real;

  begin
    x := Abs(x);
    if x < 1E-35 then
      Power := 0
    else
      begin {else}
        logPower := n * Ln(x);
        if (logPower < -80) then
          Power := 0
        else if (logPower > 80) then
          Power := Exp(80)
        else
          Power := Exp(logPower);
      end; {else}
  end; {Power}

{_________________________________________________}

  function Tan (theta: real): real;

    var
      c, s: real;

  begin
    c := Cos(theta);
    s := Sin(theta);
    Tan := s / c;
  end; {Tan}

{_________________________________________________}

  function CoTan (theta: real): real;

    const
      almostZero = 1E-35;

    var
      c, s: real;

  begin
    c := Cos(theta);
    s := Sin(theta);
    if Abs(s) < almostZero then
      s := Sign(s) * almostZero;
    CoTan := c / s;
  end; {CoTan}

{_________________________________________________}

  function Sign (alpha: real): integer;

  begin
    Sign := 0;
    if alpha < 0 then
      Sign := -1
    else
      Sign := 1;
  end; {Sign}

{_________________________________________________}

  function Arg (psi: complex): real;

    var
      theta: real;

  begin
    if psi.re > 0 then
      theta := Arctan(psi.im / psi.re)
    else if (psi.re < 0) and (psi.im > 0) then
      theta := pi + Arctan(psi.im / psi.re)
    else if (psi.re < 0) and (psi.im < 0) then
      theta := -pi + Arctan(psi.im / psi.re);
    Arg := theta
  end; {Arg}

{_________________________________________________}

  function TheArgument (thepoint: complex): real;

    var
      theta: real;

  begin
    if thepoint.re = 0 then
      theta := Sign(thepoint.im) * pi / 2;
    if thepoint.re <> 0 then
      theta := Arctan(thepoint.im / thepoint.re);
    if thepoint.re < 0 then
      theta := theta + Sign(thepoint.im) * pi;
    TheArgument := theta;
  end; {TheArgument}

{_________________________________________________}

  function TheModulus (apoint: complex): real;
  begin
    TheModulus := Sqrt(apoint.re * apoint.re + apoint.im * apoint.im);
  end; {TheModulus}

  {_________________________________________________}

  procedure ComplexToPolar (complexPoint: complex;
                  var polarPoint: polar);
   begin
    polarPoint.modu := TheModulus(complexPoint);
    polarPoint.argu := TheArgument(complexPoint);
  end; {ComplexToPolar}

  {_________________________________________________}

  procedure PolarToComplex (polarPoint: polar;
                  var complexPoint: complex);
  begin
    complexPoint.re := polarPoint.modu * Cos(polarPoint.argu);
    complexPoint.im := polarPoint.modu * Sin(polarPoint.argu);
  end; {PolarToComplex}

  {_________________________________________________}

  function Cmod (a: complex): real;

  begin
    Cmod := Sqrt(Sqr(a.re) + Sqr(a.im));
  end; {ComplexModulus}

{_________________________________________________}

  function CmodSqr (a: complex): real;

  begin
    CmodSqr := Sqr(a.re) + Sqr(a.im);
  end; {CmodSqr: Complex Squared Modulus}

{_________________________________________________}

  procedure ComplexAdd (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re + b.re;
    c.im := a.im + b.im;
    result := c;
  end; {ComplexAdd: Complex Addition}

{_________________________________________________}

  procedure ComplexSub (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re - b.re;
    c.im := a.im - b.im;
    result := c;
  end; {ComplexSub: Complex Subtraction }

{_________________________________________________}

  procedure ComplexMult (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := a.re * b.re - a.im * b.im;
    c.im := a.im * b.re + b.im * a.re;
    result := c;
  end; {Complex Multiplication}

{_________________________________________________}

  procedure ComplexDiv (a, b: complex;
                  var result: complex);
    var
      c: complex;
  begin
    c.re := (a.re * b.re + a.im * b.im) / (Sqr(b.re) + Sqr(b.im));
    c.im := (a.im * b.re - b.im * a.re) / (Sqr(b.re) + Sqr(b.im));
    result := c;
  end; {Complex Division}

{_________________________________________________}

  procedure ComplexSqrt (a: complex;
                  var result: complex);

    var
      b: complex;
      rho: real;

  begin
    rho := Sqrt(a.re * a.re + a.im * a.im);
    b.re := Sqrt((rho + a.re) / 2);
    b.im := Sign(a.im) * Sqrt((rho - a.re) / 2);
    result := b;
  end; {ComplexSqrt}

{_________________________________________________}

  procedure ComplexPoly (a: complexvector;
                  p: integer;
                  z: complex;
                  var g0: complex;
                  var b: complexvector);

    var
      i: integer;

  begin
    b[p - 1] := a[p];
    for i := 1 to p - 1 do
      begin
        ComplexMult(b[p - i], z, b[p - i - 1]);
        ComplexAdd(b[p - i - 1], a[p - i], b[p - i - 1]);
      end;
    ComplexMult(b[0], z, g0);
    ComplexAdd(g0, a[0], g0);
  end;

{_________________________________________________}

  procedure CpolyValue (a: complexvector;
                  p: integer;
                  z: complex;
                  var f: complex;
                  var b: complexvector);

    var
      i: integer;

  begin {CpolyValue}
    b[p - 1].re := a[p].re;
    b[p - 1].im := a[p].im;
    for i := 1 to p - 1 do
      begin
        b[p - i - 1].re := b[p - i].re * z.re - b[p - i].im * z.im + a[p - i].re;
        b[p - i - 1].im := b[p - i].im * z.re + b[p - i].re * z.im + a[p - i].im;
      end;
    f.re := b[0].re * z.re - b[0].im * z.im + a[0].re;
    f.im := b[0].im * z.re + b[0].re * z.im + a[0].im;
  end; {CpolyValue}

{_________________________________________________}

  procedure QuadraticRoots (var alpha: precisionVector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector);

    var
      discriminant, pi, radToDeg, numerator, denominator: real;

  begin
    pi := 4 * ArcTan(1.0);
    radToDeg := 180.0 / pi;

    cartRoot[1].im := 0;
    cartRoot[2].im := 0;
    discriminant := Sqr(alpha[1]) - 4 * alpha[0] * alpha[2];
    if (discriminant > 0) and (alpha[0] <> 0) then
      begin
        cartRoot[1].re := (-alpha[1] + Sqrt(discriminant)) / (2 * alpha[0]);
        cartRoot[2].re := (-alpha[1] - Sqrt(discriminant)) / (2 * alpha[0]);
      end;
    if (discriminant = 0) and (alpha[0] <> 0) then
      begin
        cartRoot[1].re := -alpha[1] / (2 * alpha[0]);
        cartRoot[2].re := cartRoot[1].re;
      end;

    polarRoot[1].modu := cartRoot[1].re;
    polarRoot[2].modu := cartRoot[2].re;
    polarRoot[1].argu := 0.0;
    polarRoot[2].argu := 0.0;

    if (discriminant < 0) and (alpha[0] <> 0) then
      begin
        cartRoot[1].re := -alpha[1] / (2 * alpha[0]);
        cartRoot[1].im := Sqrt(-discriminant) / (2 * alpha[0]);
        cartRoot[2].re := -alpha[1] / (2 * alpha[0]);
        cartRoot[2].im := cartRoot[1].im;

        polarRoot[1].modu := Sqrt(Sqr(cartRoot[1].re) + Sqr(cartRoot[1].im));
        polarRoot[2].modu := polarRoot[1].modu;

        if cartRoot[1].im > 0 then
          numerator := cartRoot[1].im
        else
          numerator := cartRoot[1].im;
        denominator := cartRoot[1].re;
        polarRoot[1].argu := Arctan(cartRoot[1].im / cartRoot[1].re);
        if denominator < 0 then
          polarRoot[1].argu := polarRoot[1].argu + pi;
        polarRoot[1].argu := polarRoot[1].argu * radToDeg;
        polarRoot[2].argu := -polarRoot[1].argu
      end;

  end; {QuadraticRoots}


  procedure LUsolve (start, n: integer;
                  var a: matrix;
                  var x, b: vector);

    type
      ivector = array[0..20] of integer;

    var
      v, w, pivot, lambda: real;
      i, j, k, pivotRow, finish: integer;
      p: ivector;
      d: vector;

  begin {LUsolve}

    finish := start + n - 1;
    for i := start to finish do
      begin {i; determine the scale factors}
        p[i] := i;
        d[i] := 0.0;
        for j := start to finish do
          if d[i] < Abs(a[i, j]) then
            d[i] := Abs(a[i, j]);
      end; {i}

    for i := start to finish - 1 do
      begin {i; begin the process of reduction}
        pivot := a[p[i], i];
        for k := i + 1 to finish do
          begin { k; search for a better pivot}
            v := Abs(pivot) / d[p[i]];
            w := Abs(a[p[k], i]) / d[p[k]];
            if v < w then
              begin  {interchange rows if a better pivot is found}
                pivot := a[p[k], i];
                pivotRow := p[k];
                p[k] := p[i];
                p[i] := pivotRow;
              end; {end interchange}
          end; {k; end the search for a pivot}

        for k := i + 1 to finish do
          begin {k; eliminate a[k, i]}
            lambda := a[p[k], i] / pivot;
            for j := i + 1 to finish do
              a[p[k], j] := a[p[k], j] - lambda * a[p[i], j];
            a[p[k], i] := lambda   {save the multiplier }
          end; {k}
      end; {i; reduction completed}

    for i := start to finish do
      begin {i; forward substitution}
        x[i] := b[p[i]];
        for j := i - 1 downto start do
          x[i] := x[i] - a[p[i], j] * x[j];
      end; {i; forward substitution}

    for i := finish downto start do
      begin {i; back substitution}
        for j := i + 1 to finish do
          x[i] := x[i] - a[p[i], j] * x[j];
        x[i] := x[i] / a[p[i], i];
      end; {i; back substitution}

  end; {LUsolve}

{__________________________________________}

  procedure Toeplitz (gamma: vector;
                  var y: longVector;
                  n, q: integer);

{This procedure uses a Cholesky decomposition to find the solution of}
{the equation Gx = y,  where  G is a symmetric Toeplitz matrix of order}
{n with q supra-diagonal and q sub-diagonal bands . The coefficients of}
{G are contained in gamma. The RHS vector y contains the elements of}
{the solution vector x on completion.}

    var
      t, j, k: integer;
      mu: array[0..maxOrder, 0..maxArray] of real; {wideMatrix}

  begin {Toeplitz}

{Factorise}
    for t := 0 to q - 1 do
      for j := t + 1 to q do
        mu[j, t] := 0.0;

    for t := 0 to n do
      begin {t}
        for k := Min(q, t) downto 0 do
          begin {k}
            mu[k, t] := gamma[k];
            for j := 1 to q - k do
              mu[k, t] := mu[k, t] - mu[j, t - k] * mu[j + k, t] * mu[0, t - k - j];
            if k > 0 then
              mu[k, t] := mu[k, t] / mu[0, t - k];
          end;{k}
      end; {t}

{ForwardSolve;}
    for t := 0 to n do
      for j := 1 to Min(t, q) do
        y[t] := y[t] - mu[j, t] * y[t - j];

{Divide by the diagonal}
    for t := 0 to n do
      y[t] := y[t] / mu[0, t];

{BackSolve;}
    for t := n downto 0 do
      for j := 1 to Min(q, n - t) do
        y[t] := y[t] - mu[j, t + j] * y[t + j];

  end; {Toeplitz}

{_________________________________________________}

  procedure Minit (var mu: vector;
                  var varEpsilon: real;
                  gamma: vector;
                  q: integer);

    var
      D: matrix;
      delta, f: vector;
      i, j, iterations: integer;
      convergence: boolean;


{-------------------------------------------}

    function CheckDelta (q: integer;
                    var delta, mu: vector): boolean;

      var
        i: integer;
        muNorm, deltaNorm: real;

    begin
      muNorm := 0.0;
      deltaNorm := 0.0;
      for i := 0 to q do
        begin
          muNorm := muNorm + sqr(mu[i]);
          deltaNorm := deltaNorm + Sqr(delta[i])
        end;
      if (deltaNorm / muNorm) > 0.00001 then
        CheckDelta := false
      else
        CheckDelta := true;

    end; {Checkdelta}

{------------------------------------------}

  begin {PROCEDURE Minit}

{initialise the vector mu }
    for i := 0 to q do
      mu[i] := 0.0;
    mu[0] := Sqrt(gamma[0]);
    convergence := false;
    iterations := 0;

    while (convergence = false) and (iterations < 10) do
      begin

{Form the matrix of derivatives}
        for i := 0 to q do
          for j := 0 to q do
            begin
              D[i, j] := 0.0;
              if (j - i) >= 0 then
                D[i, j] := mu[j - i];
              if (i + j) <= q then
                D[i, j] := D[i, j] + mu[i + j];
            end;

{Find the function value}
        for j := 0 to q do
          begin
            f[j] := gamma[j];
            for i := 0 to q - j do
              f[j] := f[j] - mu[i] * mu[i + j]
          end;

{Find the updating vector}
        LUsolve(0, q + 1, D, delta, f);

{Update the value of mu}
        for i := 0 to q do
          mu[i] := mu[i] + delta[i];
        iterations := iterations + 1;

{Check for convergence}
        convergence := CheckDelta(q, delta, mu);
      end; {while}
    Writeln('The number of iterations was ', iterations : 3);

{Renomalise the results}
    varEpsilon := 1.0;
    for i := 1 to q do
      begin {i}
        mu[i] := mu[i] / mu[0];
        varEpsilon := varEpsilon + mu[i] * mu[i];
      end; {i}
    mu[0] := 1;
    varEpsilon := gamma[0] / varEpsilon;

  end; {Minit}
{_________________________________________________}

end. {MATHS: Unit 2 of BLIMDOS.PAS}

