unit ORGANISE; {Unit 11 of BLIMDOS.PAS}

interface

  uses
    GLOBALS, UTILS, MATHS, DATA, ACQUIRE, AUTOREG, ROOTS, SPECTRAL, SCREEN, POST, Dos, Crt, Graph;

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure GetPseudoData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

   procedure TrendExtract (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

  procedure PlotThePeriodogram (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure PlotTheSpectrum (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure AREstimation (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure MonteCarloTrials (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

  procedure NumericalTrials (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

  procedure SaveAndExit (var dataBOX: rBOX;
                  var dataVector: longVector);

implementation

 {________________________________________________}

  procedure GetTheData (var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('    GET THE DATA');
      Writeln;
      Writeln('         1. Read the Data');
      Writeln;
      Writeln('         2. Generate Ordinary ARMA Data');
      Writeln;
      Writeln('         3. Generate Band-Limited ARMA Data');
      Writeln;
      Writeln('         4. Plot the Data');
      Writeln;
      Writeln('         5. Take Logs of the Data');
      Writeln;
      Writeln('         6. Subsample the Data');
      Writeln;
      Writeln('         7. RETURN to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press <RETURN>');

      GetBoundedInteger(1, 7, theMenuItem);

      case theMenuItem of
        1: 
          ReadTheData(dataBOX, dataVector); {in DATA}
        2: 
          GetARMAData(ARMAGen, dataBOX, dataVector); {in DATA}
        3: 
          GetBandLimitedData(ARMAGen, dataBOX, dataVector); {in DATA}
        4: 
          PlotTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in DATA}
        5: 
          LogOfVector(dataBOX, dataVector); {in DATA}
        6: 
          SubsampleTheData(dataBOX, dataVector); {in DATA}
      end;{case}
    until theMenuItem = 7;

  end;{GetTheData}

{________________________________________________}

  procedure GetPseudoData (var ARMAGen: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      bandLimited: boolean;

  begin
    ClrScr;
    Writeln;
    Writeln('    GENERATE THE ARMA DATA');
    Writeln;

    YesOrNo('    Do you wish to generate band-Limited data', bandLimited);
    if bandLimited then
      GetBandLimitedData(ARMAGen, dataBOX, dataVector) {in DATA}
    else
      begin {not bandLimited}

        if ARMAModel then
          begin
            SysBeep(3);
            YesOrNo('    You have already specified the model. Do you wish to retain it', ARMAModel);
          end;

        if (ARMAModel = false) then
          RationalAcquisition(ARMAGen, stability);

        ARMAGen.varepsilon := 1.0;
        Writeln('    What is size T of the data sample you wish to generate?');
        GetBoundedInteger(16, 500, dataBOX.NOPUNCT);
        stability := true;
        GenerateData(ARMAGen, dataBOX, dataVector);
      end; {not bandLimited}

    thereIsData := true;
    pseudoData := true;
    empiricdata := false;
    
    ClrScr;
  end; {GetPseudoData}

  {_________________________________________________}
   {_________________________________________________}

  procedure TrendExtract (var SCREENBOX, POSTBOX: IBOX;
                  var dataBOX, trendBOX, residueBOX: rBOX;
                  var dataVector, trendVector, residueVec: longVector);

    var
      theMenuItem: integer;

  begin

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

    repeat
      ClrScr;
      Writeln;
      Writeln('    FILTERING AND TREND EXTRACTION');
      Writeln;
      Writeln('         1. Extract a Polynomial Trend');
      Writeln;
      Writeln('         2. Apply a Lowpass Trigonometrical Filter');
      Writeln;
      Writeln('         3. RETURN to the Main Menu');
      Writeln;
      Writeln('                      Select a number and press RETURN');

      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1:  {in DATA}
          FitPolyTrend(dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
        2: 
          LowpassComponent(dataBOX, trendBOX, residueBOX, SCREENBOX, POSTBOX, dataVector, trendVector, residueVec);
      end;

    until theMenuItem = 3;

  end;{TrendExtract}

{________________________________________________}

  procedure PlotThePeriodogram (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      menuChoice: char;
      periodoVec, ARMASpecVec: longVector;
      plotBOX, periodoBOX, spectrumBOX: rBOX;
      postFileName, legend: string;
      F: text;

    procedure WriteOptionD;
    begin
      Writeln('        (d)  Plot the periodogram of the data');
    end;
    procedure WriteOptionG;
    begin
      Writeln('        (g)  Plot the periodogram of the data and the parametric');
      Writeln('             spectrum of the process generating the data ');
    end;
    procedure WriteOptionE;
    begin
      Writeln('        (e)  Plot the periodogram of the data and the parametric');
      Writeln('             spectrum of the model estimated from the data');
    end;

  begin {PlotThePeridoogram}
    legend := '';

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat); {in UTILS}
      end;

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    menuChoice := 'x';
    if empiricData then
      legend := 'The peridodogram of an empirical data sequence';
    if pseudoData then
      legend := 'The periodogram of the pseudo random data';

    if empiricdata and ARestimates then
      repeat
        ClrScr;
        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD;
        Writeln;
        WriteOptionE;
        Writeln;
        Writeln('    Type <d> or <e> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['e', 'E'] then
          legend := Concat(legend, ' superimposed on the parametric spectrum of an estimated model');
        if not (menuChoice in ['d', 'D', 'e', 'E']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'e', 'E'];

    if pseudoData and not ARestimates then
      repeat
        ClrScr;
        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD;
        Writeln;
        WriteOptionG;
        Writeln;
        Writeln('    Type <d>  or <g> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['G', 'g'] then
          legend := Concat(legend, ' superimposed on the spectrum of the process generating the data');
        if not (menuChoice in ['d', 'D', 'g', 'G']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'g', 'G'];

    if pseudoData and ARestimates then
      repeat
        ClrScr;
        Writeln;
        Writeln('    PLOT THE PERIODOGRAM');
        Writeln;
        WriteOptionD;
        Writeln;
        WriteOptionG;
        Writeln;
        WriteOptionE;
        Writeln;
        Writeln('    Type <d> or <e> or <g> and press <RETURN>');
        ReadAChar(menuChoice);
        if menuChoice in ['e', 'E'] then
          legend := Concat(legend, ' superimposed on the parametric spectrum of an estimated model ');
        if menuChoice in ['G', 'g'] then
          legend := Concat(legend, ' superimposed on the spectrum of the process generating the data');
        if not (menuChoice in ['d', 'D', 'e', 'E', 'g', 'G']) then
          Sysbeep(3);
      until menuChoice in ['d', 'D', 'e', 'E', 'g', 'G'];

    ClrScr;
    GetGraphics(graphicsOn, SCREENBOX, currentMode);

    FastPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec);
    plotBOX := periodoBOX;

    if menuChoice in ['G', 'g'] then
      begin {Spectrum of Generating Process}
        {Getting the spectral vectors}
        if ARMAGen.bandLimits then
          begin
            MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

            plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
            ScreenFrameLite(plotBOX, SCREENBOX);

            SetColor(red);
            SetLineStyle(0, 0, ThickWidth);
            ScreenBandPlot(ARMAGen, plotBOX, SCREENBOX, ARMASpecVec);
            SetLineStyle(0, 0, NormWidth);

            SetColor(yellow);
            LimitLine(SCREENBOX, ARMAGen);
          end
        else
          begin
            MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

            plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
            ScreenFrameLite(plotBOX, SCREENBOX);

            SetColor(red);
            SetLineStyle(0, 0, ThickWidth);
            ScreenPlotMod(plotBOX, SCREENBOX, ARMASpecVec);
            SetColor(yellow);
            SetLineStyle(0, 0, NormWidth);
          end;
      end; {Spectrum of Generating Process}

    if menuChoice in ['E', 'e'] then
      begin {Estimated Spectrum}
        MakeAREstimSpec(AREstim, SCREENBOX, ARMASpecVec, spectrumBOX);

        plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
        ScreenFrameLite(plotBOX, SCREENBOX);

        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
        ScreenPlotMod(plotBOX, SCREENBOX, ARMASpecVec);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);
      end; {Estimated Spectrum}

    if menuChoice in ['x', 'd', 'D'] then
      ScreenFrameLite(plotBOX, SCREENBOX);

    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
    ScreenPlot(plotBOX, SCREENBOX, periodoVec);
    ZeroPiAxis(SCREENBOX);
    ScreenCaption(legend);

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if PSPict then
      begin {Make a PostScript File}
        Write('    Name the PostScript File = ');
        Readln(postFileName);        
        if postFileName <> '' then

          begin {postFileName}
            Assign(F, postFileName);          
            ReWrite(F);
            PostHeader(F, POSTBOX, TeXFormat);

            if menuChoice in ['x', 'd', 'D'] then
              begin
                PostFrameLite(F, periodoBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                PostPlot(F, plotBOX, POSTBOX, periodoVec);
              end;

            if menuChoice in ['G', 'g'] then
              begin {Spectrum of Generating Process}
                if ARMAGen.bandLimits then
                  begin {Special Provisions for POST}
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectrumBOX);

                    plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                    PostFrameLite(F, plotBOX, POSTBOX);
                    PostZeroPiAxis(F, POSTBOX);

                    plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                    PostBandPlot(F, ARMAGen, plotBOX, POSTBOX, ARMASpecVec);
                    PostLimitLine(F, POSTBOX, ARMAGen);

                    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                    PostPlot(F, plotBOX, POSTBOX, periodoVec);
                  end
                else
                  begin
                    {ARMASpecVec <-- MakeScreenGenSpec (MakeDataGenSpec): both SCREEN and POST}

                    plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                    PostFrameLite(F, plotBOX, POSTBOX);
                    PostZeroPiAxis(F, POSTBOX);

                    plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                    PostPlotMod(F, plotBOX, POSTBOX, ARMASpecVec);

                    plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                    PostPlot(F, plotBOX, POSTBOX, periodoVec);
                  end;
              end; {Spectrum of Generating Process}

            if menuChoice in ['E', 'e'] then
              begin {Estimated Spectrum}
                {ARMASpecVec <--MakeAREstimSpec: both SCREEN and POST}

                plotBOX.yMax := Rmax(periodoBOX.yMax, spectrumBOX.yMax);
                PostFrameLite(F, plotBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);

                plotBOX.NOPUNCT := spectrumBOX.NOPUNCT;
                PostPlotMod(F, plotBOX, POSTBOX, ARMASpecVec);

                plotBOX.NOPUNCT := periodoBOX.NOPUNCT;
                PostPlot(F, plotBOX, POSTBOX, periodoVec);
              end; {Estimated Spectrum}

            PostFooter(F, POSTBOX, legend, TexFormat);
            Close(F);
          end; {postFileName}
      end; {Make a PostScript File}

  end; {PlotThePeridogram}

{________________________________________________}

  procedure PlotTheSpectrum (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      ARMASpecVec, AREstSpecVec, periodoVec: longVector;
      generBOX, estimBOX, spectraBOX, periodoBOX: rBOX;
      legend, postFileName: string;
      F: text;

  begin

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

    if (ARMAModel) and (ARestimates = false) then
      begin {(M1, E0)}
        ClrScr;
        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You have specified an ARMA model that may be used to generate');
        Writeln('    pseudo-random data. The spectrum of the  model is displayed.');
        Writeln;
        Writeln(' ' : 57, '<RETURN>');
        Readln;
        legend := 'The parametric spectrum of a model specified by the user';
      end; {(M1, E0)}

    if ((ARMAModel = false) and (ARestimates = false)) then
      begin {(M0, E0) --> (M1, E0)}
        ClrScr;
        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You have not specified an ARMA model and you have not estimated');
        Writeln('    any parameters. You are asked to specify an ARMA model of which');
        Writeln('    the spectral density function will be displayed.');
        Writeln;
        Writeln(' ' : 59, '<RETURN>');
        Readln;
        legend := 'The parametric spectrum of a model specified by the user';
        RationalAcquisition(ARMAGen, stability);
        ARMAGen.varepsilon := 1.0;
        ARMAModel := true;
      end; {(M0, E0) --> (M1, E0)}

    if ARMAModel and not ARestimates then
      begin {M1, E0}
{Getting the spectral vectors}
        if ARMAGen.bandLimits then
          MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX)
        else
          MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);

{Plotting}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);
        
        ScreenFrameLite(generBOX, SCREENBOX);
        ZeroPiAxis(SCREENBOX);

        if ARMAGen.bandLimits then
          ScreenBandPlot(ARMAGen, generBOX, SCREENBOX, ARMASpecVec)
        else
          ScreenPlotMod(generBOX, SCREENBOX, ARMASpecVec);
        legend := 'The parametric spectrum of a model specified by the user';
        ScreenCaption(legend);

{do the PostScript}
       if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                Rewrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, generBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                if ARMAGen.bandLimits then
                  begin
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);
                    PostBandPlot(F, ARMAGen, generBOX, POSTBOX, ARMASpecVec);
                  end
                else
                  PostPlotMod(F, generBOX, POSTBOX, ARMASpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}

      end; {M1, E0}

    if ARMAModel and ARestimates then
      begin {M1, E1}
        ClrScr;
        Writeln;
        Writeln('    PLOT THE SPECTRUM');
        Writeln;
        Writeln('    You  have estimated  an AR model from  pseudo-random  data.');
        Writeln('    The parametric spectrum of the estimated model is displayed');
        Writeln('    together with that of the process generating the data.');
        Writeln;
        Writeln(' ' : 55, '<RETURN>');
        Readln;
{Getting the spectral vectors}
        MakeAREstimSpec(AREstim, SCREENBOX, AREstSpecVec, estimBOX);
        if ARMAGen.bandLimits then
          MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX)
        else
          MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);

{Determining the framework}
        if generBOX.yMax > estimBOX.yMax then
          spectraBOX := generBOX
        else
          spectraBOX := estimBOX;

{Plotting}
        GetGraphics(graphicsOn, SCREENBOX, currentMode);

        ScreenFrameLite(spectraBOX, SCREENBOX);
        ZeroPiAxis(SCREENBOX);

        if ARMAGen.bandLimits then
          ScreenBandPlot(ARMAGen, spectraBOX, SCREENBOX, ARMASpecVec)
        else
          ScreenPlotMod(spectraBOX, SCREENBOX, ARMASpecVec);
          
        SetColor(red);
        ScreenPlotMod(spectraBOX, SCREENBOX, AREstSpecVec);
        SetColor(yellow);

        legend := 'The specta of the estimated model (in red) and of the process generating the data';
        ScreenCaption(legend);

{Do the PostScript}
        if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                Rewrite(F);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, spectraBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                if ARMAGen.bandLimits then
                  begin
                    MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, spectraBOX);
                    PostBandPlot(F, ARMAGen, spectraBOX, POSTBOX, ARMASpecVec);
                  end
                else
                  PostPlotMod(F, spectraBOX, POSTBOX, ARMASpecVec);
                PostPlotMod(F, spectraBOX, POSTBOX, AREstSpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}
      end; {M1, E1}

    if not ARMAModel and ARestimates then
      begin {M0, E1}
        ClrScr;
        Writeln;        
        Writeln('    PLOT THE SPECTRUM'); {no Model Estimtes M0, E1}
        Writeln;
        Writeln('    You have  estimated an AR model from empirical data. The');
        Writeln('    parametric spectrum of the estimated  model is displayed');
        Writeln('    together with the periodogram of the data.');
        Writeln;
        Writeln(' ' : 52, '<RETURN>');
        Readln;

{What about the scale:  should this be passed to peridodogram?}
{peridoVec --> ARMASpecVec: to save memory}
        FastPeriodogram(dataBOX, periodoBOX, dataVector, periodoVec);
        MakeAREstimSpec(AREstim, SCREENBOX, AREstSpecVec, estimBOX);
        if estimBOX.yMax > periodoBOX.yMax then
          periodoBOX.yMax := estimBOX.yMax
        else
          estimBOX.yMax := periodoBOX.yMax;

        GetGraphics(graphicsOn, SCREENBOX, currentMode);

        ScreenFrameLite(periodoBOX, SCREENBOX);
        estimBOX.yUB := periodoBOX.yUB;
        estimBOX.yLB := 0.0;

        ZeroPiAxis(SCREENBOX);
        ScreenPlot(periodoBOX, SCREENBOX, periodoVec);
        SetColor(red);
        SetLineStyle(0, 0, ThickWidth);
        ScreenPlotMod(estimBOX, SCREENBOX, AREstSpecVec);
        SetColor(yellow);
        SetLineStyle(0, 0, NormWidth);


        legend := 'The spectum of the estimated model and the periododogram of the empirical data.';
        ScreenCaption(legend);

{Do the PostScript}
        if PSPict then
          begin {Make a PostScript File}
            CallForReturn;
            Readln;
            CloseGraph;
            RestoreCrtMode;
            ClrScr;

            Write('    Name the PostScript File = ');
            Readln(postFileName);
            if postFileName <> '' then
              begin {postFileName}
                Assign(F, postFileName);
                PostHeader(F, POSTBOX, TeXFormat);
                PostFrameLite(F, periodoBOX, POSTBOX);
                PostZeroPiAxis(F, POSTBOX);
                PostPlot(F, periodoBOX, POSTBOX, periodoVec);
                PostPlotMod(F, periodoBOX, POSTBOX, AREstSpecVec);
                PostFooter(F, POSTBOX, legend, TexFormat);
                Close(F);
              end; {postFileName}
          end; {Make a PostScript File}
      end; {M0, E1}

    if not PSPict then
      begin {clear the Graphics}
        CallForReturn;
        Readln;
        CloseGraph;
        RestoreCrtMode;
        ClrScr;
      end;

  end; {PlotTheSpectrum}

{_________________________________________________}

  procedure AREstimation (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);
    var
      pacv, aCovar: vector;
      lags, pHat, t, i: integer;
      cirCovar: longVector;
      theChoice: char;

  begin
{if noPageParameters then}
{begin}
{SysBeep(3);}
{PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);}
{end;}

    if thereIsData = false then
      GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}

    if thereIsData then
      begin {thereIsData}
        ClrScr;

        Writeln;
        Writeln('    AUTOREGRESSIVE ESTIMATION');
        Writeln;
        Writeln;
        Writeln('    What is the order of the estimated autoregressive model?');
        GetBoundedInteger(0, 15, pHat);
        AREstim.p := pHat;
        lags := pHat;

        Writeln;
        repeat
          Writeln('    Type (c) for circular autocovariances (a) for ordinary autocovariances');
          ReadAChar(theChoice);
        until theChoice in ['c', 'C', 'a', 'A'];

        if theChoice in ['a', 'A'] then
          AutoCovariances(dataBOX.NOPUNCT, lags, dataVector, aCovar)
        else
          begin {Circular Autocovariances}
            CircularACovars(dataBOX.NOPUNCT, dataBOX.NOPUNCT, dataVector, cirCovar);
            for t := 0 to pHat do
              aCovar[t] := cirCovar[t];
          end; {Circular Autocovariances}

{we need to specify the order of the model}
        LevinsonDurbin(aCovar, AREstim, pacv);

        Writeln('    THE YULE--WALKER ESTIMATES');
        Writeln;
        for i := 0 to pHat do
          Writeln('    alpha[', i : 1, '] = ', AREstim.alpha[i] : 6 : 4);
        Writeln;
        Writeln(' ' : 60, '<RETURN>');
        Readln;
      end; { thereIsData}

    ClrScr;

  end; {AREstimation}

  {________________________________________________}

  procedure MonteCarloTrials (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var SCREENBOX, POSTBOX: IBOX;
                  var dataVector: longVector);

    var
      generBOX, estimBOX, specBOX: rBOX;
      trialCount, nofTrials, lags, pHat: integer;
      pacv, aCovar: vector;
      ARMASpecVec, AREstSpecVec: longVector;
      maxSpec: real;
      postFileName, legend: string;
      F: text;
      bandLimited, reScale: boolean;

    label
      999;

  begin

    if noPageParameters then
      begin
        SysBeep(3);
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);
      end;

{In this segment, we generate data accoding to a specific autoregessive model}
{and we fit an model by Yule-Walker estimation. To determine the performance,}
{we plot the parametric spectra implied by the esimated models}

    ClrScr;
    Writeln;
    Writeln('    GRAPHICAL TRIALS');
    Writeln;
    Writeln('    This segment of the program plots the  parametric spectra of a succession');
    Writeln('    of  estimated AR models that are  obtained from  independent sequences of');
    Writeln('    pseudo random data  generated by a specified  ARMA  model. The parametric');
    Writeln('    spectrum of the model generating the data is also plotted,');
    Writeln;
    Writeln('    A  provision has  been  made for generating band-limited  data sequences.');
    Writeln('    These are obtained  by sampling the continuous functions corresponding to');
    Writeln('    ordinary ARMA processes  at rates in exess of the unit sampling rate. The');
    Writeln('    continuous functions are formed  via  a process of interpolation based on');
    Writeln('    the Fourier ordinates of the discrete pseudo random  data sequence. There');
    Writeln('    is also a provision for adding a white noise contamination to the data.');
    Writeln;
    Writeln(' ' : 69, '<RETURN>');
    Readln;

    YesOrNo('    Do your wish to generate band-limited data', bandLimited);

    if bandLimited then
      GetBandLimitedData(ARMAGen, dataBOX, dataVector) {in DATA}
    else
      GetARMAData(ARMAGen, dataBOX, dataVector);

    ClrScr;
    Writeln;
    Writeln('    MONTE CARLO TRIALS');
    Writeln;
    Writeln('    What is the order of the estimated autoregressive model?');
    GetBoundedInteger(0, 15, pHat);
    AREstim.p := pHat;
    lags := pHat;
    Writeln;
    Writeln('    How many trials do you wish to perform?');
    GetBoundedInteger(1, 20, nofTrials);

    maxSpec := 0.0;
    reScale := false;
    seedStore := genSeed;

{If reScale is set to true after the first round, then the simulation results}
{will be plotted afresh to a new scale that fits the bounding box. The second}
{round will be initiated with the same random seed as the first round, which}
{has been saved in seedStore for this purpose}

999: {back to here if reScale = true}

    if reScale = true then
      genSeed := seedStore;

    postFileName := '';
    if PSPict then {make a PostScript File}
      Write('    Name the PostScript File = ');
    Readln(postFileName);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++}

{Plot the Spectum of the model generating the data}
    if bandLimited then
      MakeScreenBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX)
    else
      MakeScreenGenSpec(ARMAGen, SCREENBOX, ARMASpecVec, generBOX);

    {Find the scale for the bounding box}
    GetGraphics(graphicsOn, SCREENBOX, currentMode);     {THE PROBLEM IS HERE}

    if reScale then
      generBOX.ymax := Rmax(maxSpec, generBOX.ymax / 1.5)
    else
      generBOX.ymax := 1.5 * generBOX.ymax;

    ScreenFrameLite(generBOX, SCREENBOX);
    ZeroPiAxis(SCREENBOX);

    if bandLimited then
      begin
        LimitLine(SCREENBOX, ARMAGen);
        ScreenBandPlot(ARMAGen, generBOX, SCREENBOX, ARMASpecVec)
      end
    else
      ScreenPlotMod(generBOX, SCREENBOX, ARMASpecVec);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++}

    if postFileName <> '' then
      begin {postFileName}
        Assign(F, postFileName);
        ReWrite(F);
        PostHeader(F, POSTBOX, TeXFormat);
        Writeln(F, '0.5 setlinewidth');
{PostFrameLite(F, generBOX, POSTBOX);}
{PostZeroPiAxis(F, POSTBOX);}
        Writeln(F, '0.5 setgray');
      end; {postFileName}

    trialCount := 0;
    SetColor(red);

{RepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeat}
    repeat

      if bandLimited then
        GenerateBandLimitedData(ARMAGen, dataBOX, dataVector) {in AUTOREG}
      else
        GenerateData(ARMAGen, dataBOX, dataVector); {in AUTOREG}

      AutoCovariances(dataBOX.NOPUNCT, lags, dataVector, aCovar);
      LevinsonDurbin(aCovar, AREstim, pacv);

      MakeAREstimSpec(AREstim, SCREENBOX, AREstSpecVec, estimBOX);
      maxSpec := Rmax(estimBOX.yMax, maxSpec);

      ScreenPlotMod(generBOX, SCREENBOX, AREstSpecVec);

      if postFileName <> '' then
        begin
          Writeln(F, 'newpath');
          PostPlotMod(F, generBOX, POSTBOX, AREstSpecVec);
        end;

      trialCount := trialCount + 1;
    until (trialCount = nofTrials);
{RepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeatRepeat}

    SetColor(yellow);
    SetLineStyle(0, 0, ThickWidth);
    if bandLimited then
      ScreenBandPlot(ARMAGen, generBOX, SCREENBOX, ARMASpecVec)
    else
      ScreenPlotMod(generBOX, SCREENBOX, ARMASpecVec);
    SetColor(yellow);
    SetLineStyle(0, 0, NormWidth);

    legend := 'The spectrum of the data generating process superimposed on';
    legend := Concat(legend, ' specta estimated from pseudo-random data');
    ScreenCaption(legend);

    if postFileName <> '' then
      begin
        Writeln(F, '0.0 setgray');
        Writeln(F, '1.0 setlinewidth');
        if bandLimited then
          begin
            MakePostBandSpec(ARMAGen, SCREENBOX, ARMASpecVec, specBOX);
            specBOx.yUB := generBOX.yUB;
            PostBandPlot(F, ARMAGen, specBOX, POSTBOX, ARMASpecVec);
            PostLimitLine(F, POSTBOX, ARMAGen);
          end
        else
          PostPlotMod(F, generBOX, POSTBOX, ARMASpecVec);

        PostFrameLite(F, generBOX, POSTBOX);
        PostZeroPiAxis(F, POSTBOX);

        PostFooter(F, POSTBOX, legend, TexFormat);
        Close(F);
      end;

    CallForReturn;
    Readln;
    CloseGraph;
    RestoreCrtMode;
    ClrScr;

    if reScale = false then
      begin
        WRITELN('    maxSpec = ', maxSpec : 6 : 4);
        Writeln;
        YesOrNo('    Do you wish to rescale the bounding Box', reScale);
{if we choose to reScale, then we seed the random number generator with seedStore}
        if reScale then
          goto 999;
      end;

  end; {MonteCarloTrials}

{________________________________________________}

  procedure ModelAnalysis (var ARMAGen, AREstim: ARMARecord);

    var
      BLalpha, BLcart: complexVector;
      BLpolar: polarVector;
      j, g: integer;
      effModu, effArgu: real;

  begin
    ClrScr;
    Writeln;
    Writeln('    ANALYSIS OF ESTIMATES');
    Writeln;
    Writeln('    The band limited data have an upper bound in frequency of theta');
    Writeln('    radians per unit sampling period, which is less than the Nyquist');
    Writeln('    value of pi radians that characterises ordinary ARMA processes.');
    Writeln;
    Writeln('    The band-limited  data are generated by resampling a continuous');
    Writeln('    trajectory  corresponding  to  an ordinary ARMA  process at the');
    Writeln('    higher rate of pi/theta observations  per unit period and then');
    Writeln('    by placing the  resampled observations at unit time intervals.');
    Writeln;
    Writeln(' ' : 60, '<RETURN>');
    Readln;

    Writeln;
    Writeln('    THE VALUES OF THE ESTIMATED PARAMETERS');
    Writeln;
    for j := 0 to AREstim.p do
      begin
        BLalpha[j].re := AREstim.alpha[j];
        BLalpha[j].im := 0.0;
        Writeln('    alpha[', j : 1, '] = ', AREstim.alpha[j] : 6 : 4);
      end;
    Writeln;
    Writeln('    varEpsilon = ', AREstim.varEpsilon : 6 : 4);

{FindRoots requires BLalpha to be a complex vector }
    FindRoots(AREstim.p, BLalpha, BLcart, BLpolar);

    ClrScr;
    Writeln;
    Writeln('    THE POLES OF THE AVERAGE ESTIMATED AR MODEL');
    Writeln;
    for j := 1 to AREstim.p do
      Writeln('    poles[', j : 1, '].re = ', BLcart[j].re : 5 : 5, '   poles[', j : 1, '].im = ', BLcart[j].im : 5 : 5);
    Writeln;
    for j := 1 to AREstim.p do
      begin
        Writeln('    root(', j : 1, ').modu = ', BLpolar[j].modu : 8 : 4);
        Writeln('    root(', j : 1, ').argu = ', BLpolar[j].argu : 8 : 4);
        Writeln;
      end;

    Writeln(' ' : 59, '<RETURN>');
    Readln;

    if ARMAGen.bandLimits then
      begin {bandLimits}
        ClrScr;
        Writeln;
        Writeln;
        Writeln('    THE EFFECTIVE PARAMETERS OF THE BAND-LIMITED PROCESS');
        Writeln;
        Writeln('    The parameters of the process that  has generated the data were');
        Writeln('    specified on  the  assumption that it was  an ordinary  process');
        Writeln('    supported on  the full  Nyquist interval. The arguments and the');
        Writeln('    the moduli of the complex autoregressive roots may be converted');
        Writeln('    to the values that are relevant to the band-limited process.');
        Writeln;
        g := 1;
        for j := 1 to ARMAGen.pr do
          begin
            effModu := Ln(ARMAGen.polarPoles[g].modu) * ARMAGen.omega2Deg;
            effModu := effModu / 180.0;
            effModu := Exp(effModu);
            effArgu := (ARMAGen.polarPoles[g].argu * ARMAGen.omega2Deg) / 180.0;
            Writeln('    The effective argument  = ', effArgu : 6 : 4);
            Writeln('    True effective molulus  = ', effModu : 6 : 4);
            g := g + 2;
          end;
        Readln;
      end; {bandLimits}
    Writeln(' ' : 59, '<RETURN>');
    Readln;

  end; {ModelAnalysis}

{________________________________________________}

  procedure NumericalTrials (var ARMAGen, AREstim: ARMARecord;
                  var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      j, trialCount, nofTrials, lags: integer;
      pacv, aCovar: vector;
      alphaStore, realStore, complexStore: precisionVector;
      BLcart: complexVector;
      BLpolar: polarVector;
      bandLimited: boolean;
      discriminant: real;
      varepsilonStore: double;
      realCount, complexCount: integer;
      logFileName: string;
      Q: text;

  begin
    ClrScr;
    Writeln;

    Writeln('   NUMERICAL TRIALS');
    Writeln;
    Writeln('   The purpose of these trials is to estimate the mean values of the estimated');
    Writeln('   parameters  of autoregressive models that  are fitted to the  pseudo random');
    Writeln('   data. A model is  specified for generating  the data. This is an ARMA model');
    Writeln('   that may  be defined on a subset of the Nyquist interval that runs from the');
    Writeln('   zero frequency to some limiting value up to the Nyquist value of pi radians');
    Writeln('   per sample interval at  most. There  is also a provison for adding a  white');
    Writeln('   noise contamination to the data.');
    Writeln(' ' : 70, '<RETURN>');
    Readln;
    YesOrNo('    Do your wish to generate band-limited data', bandLimited);

{A model will be called for if one has not already been specified}
    if bandLimited then
      GetBandLimitedData(ARMAGen, dataBOX, dataVector) {in DATA}
    else
      GetARMAData(ARMAGen, dataBOX, dataVector); {in DATA}

    Writeln('    What is the order of the estimated autoregressive model?');
    GetBoundedInteger(0, 15, AREstim.p);
    lags := AREstim.p;
    Writeln;
    Writeln('    How many trials do you wish to perform?');
    GetBoundedInteger(1, 10000, nofTrials);

    Writeln('    Name the file for the simulation results');
    Readln(logFileName);

    if logFileName = '' then
      logFileName := 'SIMLOGFILE';
    Assign(Q, logFileName);
    Rewrite(Q);

    complexStore[1] := 0.0;
    complexStore[2] := 0.0;
    realStore[1] := 0.0;
    realStore[2] := 0.0;
    varEpsilonStore := 0.0;
    trialCount := 0;
    complexCount := 0;
    realCount := 0;
    for j := 0 to AREstim.p do
      alphaStore[j] := 0.0;

    repeat {until trialCount = nofTrials}

      if bandLimited then
        GenerateBandLimitedData(ARMAGen, dataBOX, dataVector) {in AUTOREG}
      else
        GenerateData(ARMAGen, dataBOX, dataVector); {in AUTOREG}

      AutoCovariances(dataBOX.NOPUNCT, lags, dataVector, aCovar);
      LevinsonDurbin(aCovar, AREstim, pacv);

      for j := 0 to AREstim.p do
        alphaStore[j] := alphaStore[j] + AREstim.alpha[j];
      varEpsilonStore := varEpsilonStore + AREstim.varEpsilon;

      if AREstim.p = 2 then
        begin {p = 2}
          discriminant := AREstim.alpha[1] * AREstim.alpha[1] - 4.0 * AREstim.alpha[0] * AREstim.alpha[2];
          if discriminant < 0.0 then
            begin
              complexCount := complexCount + 1;
              complexStore[1] := complexStore[1] + AREstim.alpha[1];
              complexStore[2] := complexStore[2] + AREstim.alpha[2];
            end
          else
            begin
              realCount := realCount + 1;
              realStore[1] := realStore[1] + AREstim.alpha[1];
              realStore[2] := realStore[2] + AREstim.alpha[2];
            end;
        end; {p = 2}

      trialCount := trialCount + 1;
    until (trialCount = nofTrials);

    for j := 0 to AREstim.p do
      begin
        alphaStore[j] := alphaStore[j] / trialCount;
        ARestim.alpha[j] := alphaStore[j]
      end;
    varEpsilonStore := varEpsilonStore / trialCount;
    ARestim.varEpsilon := varEpsilonStore;

    if (realCount > 0) then
      begin
        realStore[0] := 1.0;
        realStore[1] := realStore[1] / realCount;
        realStore[2] := realStore[2] / realCount;
      end;

    if (complexCount > 0) then
      begin
        complexStore[0] := 1.0;
        complexStore[1] := complexStore[1] / complexCount;
        complexStore[2] := complexStore[2] / complexCount;
      end;

    if (AREstim.p = 2) then
      begin {p=2}
        QuadraticRoots(alphaStore, BLcart, BLpolar);
        Writeln;
        Writeln('    THE VALUES OF THE ESTIMATED PARAMETERS');
        Writeln('    AVERAGED OVER ', trialCount : 5, ' TRIALS ');
        Writeln;
        Writeln('    alpha[0] = ', alphaStore[0] : 6 : 4);
        Writeln('    alpha[1] = ', alphaStore[1] : 6 : 4);
        Writeln('    alpha[2] = ', alphaStore[2] : 6 : 4);
        Writeln;
        QuadraticRoots(alphaStore, BLcart, BLpolar);
        Writeln('    root[1] = ', BLcart[1].re : 8 : 4, ' + ', BLcart[1].im : 8 : 4);
        Writeln('    root[2] = ', BLcart[1].re : 8 : 4, ' - ', BLcart[1].im : 8 : 4);
        Writeln;
        Writeln('    modulus = ', BLpolar[1].modu : 8 : 4);
        Writeln('    argument = ', BLpolar[1].argu : 8 : 4);
        Writeln(Q);
        Writeln(Q, '    THE VALUES OF THE ESTIMATED PARAMETERS');
        Writeln(Q, '    AVERAGED OVER ', trialCount : 5, ' TRIALS ');
        Writeln(Q);
        Writeln(Q, '    alpha[0] = ', alphaStore[0] : 6 : 4);
        Writeln(Q, '    alpha[1] = ', alphaStore[1] : 6 : 4);
        Writeln(Q, '    alpha[2] = ', alphaStore[2] : 6 : 4);
        Writeln(Q);
        QuadraticRoots(alphaStore, BLcart, BLpolar);
        Writeln(Q, '    root[1] = ', BLcart[1].re : 8 : 4, ' + ', BLcart[1].im : 8 : 4);
        Writeln(Q, '    root[2] = ', BLcart[1].re : 8 : 4, ' - ', BLcart[1].im : 8 : 4);
        Writeln(Q);
        Writeln(Q, '    modulus = ', BLpolar[1].modu : 8 : 4);
        Writeln(Q, '    argument = ', BLpolar[1].argu : 8 : 4);

        if (complexCount > 0) then
          begin {complexCount}
            QuadraticRoots(complexStore, BLcart, BLpolar);
            Writeln;
            Writeln('    THE VALUES OF THE ESTIMATED PARAMETERS');
            Writeln('    IN THE CASE OF COMPLEX ROOTS');
            Writeln('    AVERAGED OVER ', complexCount : 5, ' TRIALS ');
            Writeln;
            Writeln('    The number of cases with complex-valued roots was ', complexCount : 4);
            Writeln;
            Writeln('    alpha[0] = ', complexStore[0] : 6 : 4);
            Writeln('    alpha[1] = ', complexStore[1] : 6 : 4);
            Writeln('    alpha[2] = ', complexStore[2] : 6 : 4);
            Writeln;
            QuadraticRoots(complexStore, BLcart, BLpolar);
            Writeln('    root[1] = ', BLcart[1].re : 6 : 4, ' + ', BLcart[1].im : 6 : 4);
            Writeln('    root[2] = ', BLcart[1].re : 6 : 4, ' - ', BLcart[1].im : 6 : 4);
            Writeln;
            Writeln('    modulus = ', BLpolar[1].modu : 6 : 4);
            Writeln('    argument = ', BLpolar[1].argu : 6 : 4);

            Writeln(Q);
            Writeln(Q, '    THE VALUES OF THE ESTIMATED PARAMETERS');
            Writeln(Q, '    IN THE CASE OF COMPLEX ROOTS');
            Writeln(Q, '    AVERAGED OVER ', complexCount : 5, ' TRIALS ');
            Writeln(Q);
            Writeln(Q, '    The number of cases with complex-valued roots was ', complexCount : 4);
            Writeln(Q);
            Writeln(Q, '    alpha[0] = ', complexStore[0] : 6 : 4);
            Writeln(Q, '    alpha[1] = ', complexStore[1] : 6 : 4);
            Writeln(Q, '    alpha[2] = ', complexStore[2] : 6 : 4);
            Writeln(Q);
            Writeln(Q, '    root[1] = ', BLcart[1].re : 6 : 4, ' + ', BLcart[1].im : 6 : 4);
            Writeln(Q, '    root[2] = ', BLcart[1].re : 6 : 4, ' - ', BLcart[1].im : 6 : 4);
            Writeln(Q);
            Writeln(Q, '    modulus = ', BLpolar[1].modu : 6 : 4);
            Writeln(Q, '    argument = ', BLpolar[1].argu : 6 : 4);
          end; {complexCount}

        if (realCount > 0) then
          begin {realCount}
            Writeln;
            Writeln('    THE VALUES OF THE ESTIMATED PARAMETERS');
            Writeln('    IN THE CASE OF REAL ROOT3');
            Writeln('    AVERAGED OVER ', realCount : 5, ' TRIALS ');
            Writeln;
            Writeln('    The number of cases with real-valued roots was ', realCount : 4);
            Writeln;
            Writeln('    alpha[0] = ', realStore[0] : 6 : 4);
            Writeln('    alpha[1] = ', realStore[1] : 6 : 4);
            Writeln('    alpha[2] = ', realStore[2] : 6 : 4);
            Writeln;
            QuadraticRoots(realStore, BLcart, BLpolar);
            Writeln('    realRoot[1] = ', BLcart[1].re : 8 : 4);
            Writeln('    realpoot[2] = ', BLcart[2].re : 8 : 4);

            Writeln(Q);
            Writeln(Q, '    THE VALUES OF THE ESTIMATED PARAMETERS');
            Writeln(Q, '    IN THE CASE OF REAL ROOT3');
            Writeln(Q, '    AVERAGED OVER ', realCount : 5, ' TRIALS ');
            Writeln(Q);
            Writeln(Q, '    The number of cases with real-valued roots was ', realCount : 4);
            Writeln(Q);
            Writeln(Q, '    alpha[0] = ', realStore[0] : 6 : 4);
            Writeln(Q, '    alpha[1] = ', realStore[1] : 6 : 4);
            Writeln(Q, '    alpha[2] = ', realStore[2] : 6 : 4);
            Writeln(Q);
            Writeln(Q, '    realRoot[1] = ', BLcart[1].re : 8 : 4);
            Writeln(Q, '    realRoot[2] = ', BLcart[2].re : 8 : 4);
          end;  {realCount}

      end; {p=2}
    Close(Q);

    Writeln(' ' : 50, '<RETURN>');
    Readln;

    ModelAnalysis(ARMAGen, AREstim);

  end; {NumericalTrials}

{________________________________________________}

  procedure SaveAndExit (var dataBOX: rBOX;
                  var dataVector: longVector);

    var
      theMenuItem: integer;

  begin
    repeat
      ClrScr;
      Writeln;
      Writeln('    SAVE AND EXIT');
      Writeln;
      Writeln('      1. Save the  Data');
      Writeln;
      Writeln('      2. RETURN to the Main Menu');
      Writeln;
      Writeln('      3. EXIT the Program');
      Writeln;
      Writeln('                      Select a number and press RETURN');
      GetBoundedInteger(1, 3, theMenuItem);

      case theMenuItem of
        1: 
          WriteData(dataBOX, dataVector); {in UTILS}
        2: 
          ;{RETURN to the Main Menu}
        3: 
          allsDone := true; {EXIT the Program}
      end;{case}

    until theMenuItem in [1..3];

  end;{SaveAndExit}

{________________________________________________}

end. {ORGANISE: Unit 11 of WHITYULE.PAS}

