unit SPECTRAL; {Unit 7 of BLIMDOS.PAS}

interface

  uses
    GLOBALS, MATHS;

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

  function Spectrum (var scale: real;
                  var chi, phi: vector;
                  omega: real;
                  g, d: integer): real;

  procedure SpectrumAndOmega (var scale: real;
                  var chi, phi: vector;
                  g, d: integer;
                  var spectrumBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector);

  procedure FourierCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var FourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure FourierSynthesis (var dataBOX: rBOX;
                  var cutOffInt: integer;
                  var dataVector: longVector;
                  var fourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);

  procedure SlowPeriodogram (var dataBOX, periodoBOX: rBOX;
                  var dataVector: longVector;
                  var periodoVec: longVector);

  procedure PrimeFactors (Tcap: integer;
                  var g: integer;
                  var N: ivector;
                  var palindrome: boolean);

  function tOfj (j, g: integer;
                  P, Q: ivector): integer;

  procedure ReOrder (P, Q: ivector;
                  Tcap, g: integer;
                  var yImag, yReal: longVector);

  procedure MixedRadixCore (var yReal, yImag: longVector;
                  var N, P, Q: ivector;
                  Tcap, g: integer;
                  inverse: boolean);

  procedure MixedRadixFFT (var yReal, yImag: longVector;
                  var Tcap, g: integer;
                  inverse: boolean);

  procedure RealMixedRadix (x: longVector;
                  var xi: longVector;
                  var Ncap: integer);

  procedure Periodogram (xi: longVector;
                  var periodoVec: longVector;
                  var maxi: real;
                  Tcap: integer);

implementation

{_________________________________________________}

  procedure Laurent (var mu, gamma: vector;
                  q: integer);

    var
      i, j: integer;

  begin
    for i := 0 to q do
      begin {i}
        gamma[i] := 0.0;
        for j := 0 to q - i do
          gamma[i] := gamma[i] + mu[j] * mu[j + i];
      end; {i}
  end; {Laurent}

  {________________________________________________}

  function Spectrum (var scale: real;
                  var chi, phi: vector;
                  omega: real;
                  g, d: integer): real;

    var
      j: integer;
      numerator, denominator: extended;
{twoPi, factor: real;}

  begin
{twoPi := 8 * Arctan(1.0);}
    numerator := phi[0];
    for j := 1 to d do
      numerator := numerator + 2 * phi[j] * Cos(omega * j);
    denominator := chi[0];
    for j := 1 to g do
      denominator := denominator + 2 * chi[j] * Cos(omega * j);
{factor := varepsilon / twoPi;}
{varepsilon is replaced by a scale parameter}
    Spectrum := (numerator / denominator) * scale;
  end; {Spectrum}

{_________________________________________________}

  procedure SpectrumAndOmega (var scale: real;
                  var chi, phi: vector;
                  g, d: integer;
                  var spectrumBOX: rBOX;
                  var SCREENBOX: IBOX;
                  var spectrumVec: longVector);

    var
      i, BOXWDT: integer;
      omega, increment: real;

  begin {SpectrumAndOmega}

    BOXWDT := SCREENBOX.X2 - SCREENBOX.X1;

    spectrumBOX.xMax := pi; {the maximum value of x in the data vector}
    spectrumBOX.xMin := 0.0; {the minmum value  of x in the data vector}
    spectrumBOX.xLB := spectrumBOX.xMin;
    spectrumBOX.xRB := spectrumBOX.xMax;
    spectrumBOX.yMax := 0.0; {the maximum value of y in the Data vector}
    spectrumBOX.yMin := 0.0; {the minimum value of y in the Data vector}
    spectrumBOX.NOPUNCT := BOXWDT;

    omega := 0.0;
    increment := pi / BOXWDT;

    for i := 0 to BOXWDT do
      begin {i}
        spectrumVec[i] := Spectrum(scale, chi, phi, omega, g, d);
        if spectrumVec[i] > spectrumBOX.yMax then
          spectrumBOX.yMax := spectrumVec[i];
        omega := omega + increment;
      end;

  end; {SpectrumAndOmega}

{_________________________________________________}

  procedure FourierCoefficients (var dataBOX: rBOX;
                  var dataVector: longVector;
                  var FourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      t, j, sign: integer;
      omegaj, omegajt: real;
      Ncap: integer;

  begin
    Ncap := dataBOX.NOPUNCT div 2;
    alpha[0] := 0.0;
    beta[0] := 0.0;
    alpha[Ncap] := 0.0;

    for t := 0 to dataBOX.NOPUNCT - 1 do
      alpha[0] := alpha[0] + dataVector[t];
    alpha[0] := alpha[0] / dataBOX.NOPUNCT;

    sign := 1;
    if Even(dataBOX.NOPUNCT) then
      begin {if}
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin {t}
            alpha[Ncap] := alpha[Ncap] + sign * dataVector[t];
            sign := -sign
          end; {t}
        alpha[Ncap] := alpha[Ncap] / dataBOX.NOPUNCT;
      end {if}
    else
      Ncap := Ncap + 1;

    for j := 1 to NCap - 1 do
      begin
        omegaj := FourierInc * j;
        alpha[j] := 0.0;
        beta[j] := 0.0;
        for t := 0 to dataBOX.NOPUNCT - 1 do
          begin
            omegajt := omegaj * t;
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
          end;
        alpha[j] := alpha[j] * 2 / dataBOX.NOPUNCT;
        beta[j] := beta[j] * 2 / dataBOX.NOPUNCT;
      end;

  end; {FourierCoefficients}

  {_________________________________________________}

  procedure FourierSynthesis (var dataBOX: rBOX;
                  var cutOffInt: integer;
                  var dataVector: longVector;
                  var fourierInc: real;
                  var alpha: fourierVec;
                  var beta: fourierVec);

    var
      t, j: integer;
      omegat, omegajt, yMax, yMin: real;

  begin
    yMax := 0.0;
    yMin := 0.0;

    for t := 0 to dataBOX.NOPUNCT - 1 do
      begin {t}
        dataVector[t] := alpha[0];
        omegat := fourierInc * t;
        for j := 1 to cutOffInt do
          begin {j}
            omegajt := omegat * j;
            dataVector[t] := dataVector[t] + alpha[j] * Cos(omegajt);
            dataVector[t] := dataVector[t] + beta[j] * Sin(omegajt);
          end; {j}
        yMax := Rmax(ymax, dataVector[t]);
        yMin := Rmin(ymin, dataVector[t]);
      end;{t}

    dataBOX.yMax := yMax;
    dataBOX.yMin := yMin;
    dataBOX.xMax := dataBOX.NOPUNCT - 1;  {the maximum value of x in the dataVector}
    dataBOX.xMin := 0.0;   {the minimum value of x in the dataVector}
    dataBOX.xLB := dataBOX.xMin;   {xLB <= xMin: a left (lower) bound for x}
    dataBOX.xRB := dataBOX.xMax;    {xRB >= xMax: a right (upper) bound for x}

  end; {FourierSynthesis}

{_________________________________________________}

  procedure SlowPeriodogram (var dataBOX, periodoBOX: rBOX;
                  var dataVector: longVector;
                  var periodoVec: longVector);

    var
      alpha, beta: longVector;
      j, t, Tcap, Ncap: integer;
      pi, fourierInc, omegaj, omegajt, scale: double;
      maxi: real;
{The data should be mean adjusted, In which case alpha[0] = beta[0] = 0.}

  begin
    Tcap := dataBOX.NOPUNCT;
    Ncap := Tcap div 2;
    pi := 4.0 * Arctan(1.0);
    scale := 1 / (2.0 * Tcap * pi);
    fourierInc := (2 * pi) / Tcap;
    maxi := 0.0;

    omegaj := 0.0;
    for j := 0 to Ncap do
      begin {j}
        alpha[j] := 0.0;
        beta[j] := 0.0;
        omegajt := 0.0;
        for t := 0 to Tcap - 1 do
          begin {t}
            alpha[j] := alpha[j] + Cos(omegajt) * dataVector[t];
            beta[j] := beta[j] + Sin(omegajt) * dataVector[t];
            omegajt := omegajt + omegaj;
          end; {t}
        periodoVec[j] := scale * (Sqr(alpha[j]) + Sqr(beta[j]));
        if j > 0 then
          maxi := Rmax(maxi, periodoVec[j]);
        omegaj := omegaj + fourierInc;
      end; {j}

    periodoVec[0] := Sqr(alpha[0]) * scale;
    if Tcap = 2 * Ncap then
      begin {if}
        periodoVec[Ncap] := Sqr(alpha[Ncap]) * scale;
      end; {if}
    periodoVec[0] := 0.0; {This is tantamount the mean-adjustment of the data}

    periodoBOX.xMax := pi;
    periodoBOX.xMin := 0;
    periodoBOX.yMax := maxi;
    periodoBOX.yMin := 0;
    periodoBOX.NOPUNCT := Ncap; {Ncap +1?}

  end; {SlowPeriodogram}

{________________________________________________}

  procedure PrimeFactors (Tcap: integer;
                  var g: integer;
                  var N: ivector;
                  var palindrome: boolean);

    var
      i, p, T, first, last, store: integer;

    function Next (p: integer): integer;
    begin {Next}
      if p = 2 then
        Next := 3
      else
        Next := p + 2
    end; {Next}

  begin {PrimeFactors}
    palindrome := false;
    g := 1;
    p := 2;
    T := Tcap;

{Find the prime factors}
    while sqr(p) <= T do
      begin {while}
        if T mod p = 0 then
          begin
            T := T div p;
            N[g] := p;
            g := g + 1
          end
        else
          p := Next(p);
      end;{while}
    N[g] := T;

    first := 1;
    last := g;

{Rearrange the factors}
    while (N[first] <= N[first + 1]) do
      begin {while}
        store := N[first];
        for i := first to last - 1 do
          N[i] := N[i + 1];
        N[last] := store;
        if (N[first] = N[last]) then
          first := first + 1;
        last := g + 1 - first;
      end; {while}
    if (last - first <= 0) and (g <> 1) then
      palindrome := true

  end; {PrimeFactors}

{_________________________________________________}

  function tOfj (j, g: integer;
                  P, Q: ivector): integer;

    var
      i, t: integer;

  begin
    t := (j div Q[g]) * P[g];
    for i := g - 1 downto 1 do
      t := t + ((j mod Q[i + 1]) div Q[i]) * P[i];
    tOfj := t
  end; {tOfj}

{_________________________________________________}

  procedure ReOrder (P, Q: ivector;
                  Tcap, g: integer;
                  var yImag, yReal: longVector);

    var
      h, t, j, Pcount: integer;

    procedure Pcycle (h: integer;
                    var yImag, yReal: longVector;
                    var Pcount: integer);

      var
        j, t: integer;
        Rstore, Istore: real;

    begin {Pcycle}
      j := h;
      t := tOfj(j, g, P, Q);
      if t = j then
        Pcount := Pcount + 1
      else
        begin {else}
          Rstore := yReal[j];
          Istore := yImag[j];
          repeat
            yReal[j] := yReal[t];
            yImag[j] := yImag[t];
            Pcount := Pcount + 1;
            j := t;
            t := tOfj(j, g, P, Q);
          until t = h;
          yReal[j] := Rstore;
          yImag[j] := Istore;
          Pcount := Pcount + 1;
        end; {else}
    end; {Pcycle}

  begin {ReOrder}
    h := 1;
    Pcount := 1;
    Pcycle(h, yImag, yReal, Pcount);

    repeat {h}
      h := h + 1;
      j := h;
      repeat {j}
        t := tOfj(j, g, P, Q);
        j := t;
      until j <= h;
      if j = h then
        Pcycle(h, yImag, yReal, Pcount);
    until (h = Tcap - 1) or (Pcount = Tcap - 1);

  end; {ReOrder}

{_________________________________________________}

  procedure MixedRadixCore (var yReal, yImag: longVector;
                  var N, P, Q: ivector;
                  Tcap, g: integer;
                  inverse: boolean);

    type
      Wvector = array[0..23] of real;
      Wmatrix = array[0..23] of Wvector;

    var
      a, b, c, t, i, j, k, l, r: integer;
      W, Wl, theta, twopi: real;
      yR, yI: Wvector;
      cosine, Sine: Wmatrix;

  begin {MixedRadixCore}
    twopi := 8 * ArcTan(1.0);
    W := twopi / Tcap;
    Q[0] := 1;
    N[0] := 1;
    P[0] := Tcap;
    b := 0;
    c := 0;

    for l := 1 to g do
      begin {l : this is the major loop}
        a := 0;
        t := 0;
        Q[l] := Q[l - 1] * N[l - 1];
        P[l] := P[l - 1] div N[l];

  {Construct the transformation matrix}
        Wl := twopi / N[l];
        for j := 0 to N[l] - 1 do
          for r := j to N[l] - 1 do
            begin {r,j}
              theta := Wl * ((j * r) mod N[l]);
              if inverse then
                theta := -theta;
              cosine[j, r] := Cos(theta);
              sine[j, r] := Sin(theta);
              cosine[r, j] := cosine[j, r];
              sine[r, j] := Sine[j, r];
            end; {r, j}

        for i := 0 to Q[l] - 1 do
          begin {i}
            for k := 0 to P[l] - 1 do
              begin {k}
  {Subsequences are indexed by i,k jointly}

  {Copy a subsequence of the data}
                for j := 0 to N[l] - 1 do
                  begin {j}
                    t := a + b + c;
                    yR[j] := yReal[t];
                    yI[j] := yImag[t];
                    b := b + P[l];
                  end; {j}
                b := 0;

  {Transform the subsequence}
                for j := 0 to N[l] - 1 do
                  begin {j}
                    t := a + b + c;
                    yReal[t] := 0.0;
                    yImag[t] := 0.0;
                    for r := 0 to N[l] - 1 do
                      begin {r}
                        yReal[t] := yReal[t] + yR[r] * cosine[j, r] + yI[r] * sine[j, r];
                        yImag[t] := yImag[t] + yI[r] * cosine[j, r] - yR[r] * sine[j, r];
                      end; {r}
                    b := b + P[l];
                  end; {j}
                b := 0;

  {Scale the subsequence by the twiddle factors}
                if l < g then
                  begin {if}
                    for j := 0 to N[l] - 1 do
                      begin {j: twiddle factors}
                        t := a + b + c;
                        theta := W * ((j * c * Q[l]) mod Tcap);
                        if inverse then
                          theta := -theta;
                        yR[0] := yReal[t];
                        yI[0] := yImag[t];
                        yReal[t] := yR[0] * Cos(theta) + yI[0] * Sin(theta);
                        yImag[t] := yI[0] * Cos(theta) - yR[0] * Sin(theta);
                        b := b + P[l];
                      end; {j: twiddle factors}
                    b := 0;
                  end; {if}

                c := c + 1;
              end; {k}
            c := 0;
            a := a + N[l] * P[l];
          end; {i}
      end; {l : the major loop}

  end; {MixedRadixCore}

{_________________________________________________}

  procedure MixedRadixFFT (var yReal, yImag: longVector;
                  var Tcap, g: integer;
                  inverse: boolean);

    var
      t, i, Nmax: integer;
      Q, N, P: ivector;
      palindrome: boolean;

  begin {MixedRadixFFT}
    g := 0;
    Tcap := Tcap + 1;

    repeat {find a composite number for T}
      Tcap := Tcap - 1;
      palindrome := false;
      PrimeFactors(Tcap, g, N, palindrome);
      Nmax := 0;
      for i := 1 to g do
        if N[i] > Nmax then
          Nmax := N[i];
    until (g >= 3) and (Nmax <= 23);

    if inverse = false then
      for t := 0 to Tcap - 1 do
        begin {t}
          yReal[t] := yReal[t] / Tcap;
          yImag[t] := yImag[t] / Tcap
        end; {t}

    MixedRadixCore(yReal, yImag, N, P, Q, Tcap, g, inverse);
    ReOrder(P, Q, Tcap, g, yImag, yReal);

  end; {MixedRadixFFT}

{_________________________________________________}

  procedure RealMixedRadix (x: longVector;
                  var xi: longVector;
                  var Ncap: integer);

    var
      t, i, j, g, Nmax, Tcap: integer;
      Q, N, P: ivector;
      yReal, yImag: longVector;
      palindrome, inverse: boolean;
      zetaReven, zetaRodd, zetaIeven, zetaIodd: real;
      pi, theta, increment, sine, cosine: real;

  begin
    pi := 4 * ArcTan(1.0);
    g := 0;
    Tcap := Ncap div 2 + 1;

    repeat {find a composite number for T}
      Tcap := Tcap - 1;
      palindrome := false;
      PrimeFactors(Tcap, g, N, palindrome);

      Nmax := 0;
      for i := 1 to g do
        if N[i] > Nmax then
          Nmax := N[i];
    until (g >= 3) and (Nmax <= 23);

    Ncap := 2 * Tcap;
    increment := pi / Tcap;
    theta := 0;

    for t := 0 to Tcap - 1 do
      begin
        yReal[t] := x[2 * t] / Ncap;
        yImag[t] := x[2 * t + 1] / Ncap;
      end;

    inverse := false;
    MixedRadixCore(yReal, yImag, N, P, Q, Tcap, g, inverse);
    ReOrder(P, Q, Tcap, g, yImag, yReal);

    for j := 1 to (Tcap div 2) do
      begin{j}
        theta := theta + increment;
        cosine := Cos(theta);
        sine := Sin(theta);

        zetaReven := (yReal[j] + yReal[Tcap - j]) / 2;
        zetaRodd := (yReal[j] - yReal[Tcap - j]) / 2;
        zetaIeven := (yImag[j] + yImag[Tcap - j]) / 2;
        zetaIodd := (yImag[j] - yImag[Tcap - j]) / 2;

        xi[j] := zetaReven + cosine * zetaIeven - sine * zetaRodd;
        xi[Tcap - j] := zetaReven - cosine * zetaIeven + sine * zetaRodd;
        xi[Tcap + j] := zetaIodd - cosine * zetaRodd - sine * zetaIeven;
        xi[Ncap - j] := -zetaIodd - cosine * zetaRodd - sine * zetaIeven;
      end;{j}
    xi[0] := yReal[0] + yImag[0];
    xi[Tcap] := yReal[0] - yImag[0];

  end; {RealMixedRadix}

{_________________________________________________}

  procedure Periodogram (xi: longVector;
                  var periodoVec: longVector;
                  var maxi: real;
                  Tcap: integer);

    var
      j, NCap: integer;
      omega, pi, increment, scale: real;

  begin
    pi := 4.0 * ArcTan(1.0);
    NCap := Tcap div 2;
{scale := 2 * Tcap;}
{the scale for the periodogram}
    scale := Tcap / (2 * pi);
{the scale for the sample spectrum}
    increment := 2 * pi / Tcap;
    omega := 0.0;
    periodoVec[0] := 0.0;
{periodoVec[0] := scale * Sqr(xi[0]);}
{maxi := Rmax(periodoVec[0], periodoVec[NCap]); }
    maxi := 0.0;
    for j := 1 to NCap - 1 do
      begin {j}
        omega := omega + increment;
        periodoVec[j] := scale * (Sqr(xi[j]) + Sqr(xi[Ncap + j]));
        maxi := Rmax(maxi, periodoVec[j]);
      end; {j}
    periodoVec[NCap] := scale * Sqr(xi[NCap]);

  end; {Periodogram}

{________________________________________________}

end. {SPECTRAL: Unit 7 of BLIMDOS.PAS}

